@@ -1,16 +1,16 @@
 /*
- * Copyright 2005-2006 Gentoo Foundation
+ * Copyright 2005-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/dumpelf.c,v 1.20 2006/12/11 03:31:54 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/dumpelf.c,v 1.23 2007/08/20 09:54:15 vapier Exp $
  *
- * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-#include "paxinc.h"
+static const char *rcsid = "$Id: dumpelf.c,v 1.23 2007/08/20 09:54:15 vapier Exp $";
+const char * const argv0 = "dumpelf";
 
-static const char *rcsid = "$Id: dumpelf.c,v 1.20 2006/12/11 03:31:54 vapier Exp $";
-#define argv0 "dumpelf"
+#include "paxinc.h"
 
 /* prototypes */
 static void dumpelf(const char *filename, long file_cnt);
@@ -43,6 +43,8 @@ static void dumpelf(const char *filename, long file_cnt)
 	if ((elf = readelf(filename)) == NULL)
 		return;
 
+	printf("#include <elf.h>\n");
+
 	printf(
 		"\n"
 		"/*\n"
@@ -332,7 +334,7 @@ static void parseargs(int argc, char *argv[])
 
 	{
 	long file_cnt = 0;
-	printf("#include <elf.h>\n");
+
 	while (optind < argc)
 		dumpelf(argv[optind++], file_cnt++);
 	}
