@@ -1,16 +1,14 @@
 /*
- * Copyright 2003-2006 Gentoo Foundation
+ * Copyright 2003-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.48 2006/02/18 15:51:11 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.56 2007/08/20 09:54:15 vapier Exp $
  *
- * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #include "paxinc.h"
 
-#define argv0 "paxelf"
-
 /*
  * Setup a bunch of helper functions to translate
  * binary defines into readable strings.
@@ -100,7 +98,7 @@ int get_etype(elfobj *elf)
 		type = EGET(EHDR32(elf->ehdr)->e_type);
 	else
 		type = EGET(EHDR64(elf->ehdr)->e_type);
-        return type;
+	return type;
 }
 
 const char *get_elfetype(elfobj *elf)
@@ -108,12 +106,27 @@ const char *get_elfetype(elfobj *elf)
 	return find_pairtype(elf_etypes, get_etype(elf));
 }
 
+const char *get_endian(elfobj *elf)
+{
+	if (elf->data[EI_DATA] == ELFDATA2LSB)
+		return (char *) "LE";
+	if (elf->data[EI_DATA] == ELFDATA2MSB)
+		return (char *) "BE";
+	return (char *) "??";
+}
+
 void print_etypes(FILE *stream)
 {
-        int i;
+	int i, wrap = 0;
 	for (i = 0; elf_etypes[i].str; ++i) {
-		fprintf(stream, " (%d) = %s\n", elf_etypes[i].value, elf_etypes[i].str);
+		fprintf(stream, " (%4x) = %-10s", elf_etypes[i].value, elf_etypes[i].str);
+		if (++wrap >= 4) {
+			fprintf(stream, "\n");
+			wrap = 0;
+		}
 	}
+	if (wrap)
+		fprintf(stream, "\n");
 }
 
 int etype_lookup(const char *str)
@@ -205,15 +218,43 @@ static pairtype elf_emtypes[] = {
 	QUERY(EM_OPENRISC),
 	QUERY(EM_ARC_A5),
 	QUERY(EM_XTENSA),
+	QUERY(EM_VIDEOCORE),
+	QUERY(EM_TMM_GPP),
+	QUERY(EM_NS32K),
+	QUERY(EM_TPC),
+	QUERY(EM_SNP1K),
+	QUERY(EM_ST200),
+	QUERY(EM_IP2K),
+	QUERY(EM_MAX),
+	QUERY(EM_CR),
+	QUERY(EM_F2MC16),
+	QUERY(EM_MSP430),
+	QUERY(EM_BLACKFIN),
+	QUERY(EM_SE_C33),
+	QUERY(EM_SEP),
+	QUERY(EM_ARCA),
+	QUERY(EM_UNICORE),
 	QUERY(EM_NUM),
 	QUERY(EM_ALPHA),
 	{ 0, 0 }
 };
-const char *get_elfemtype(int type)
+
+int get_emtype(elfobj *elf)
 {
-	return find_pairtype(elf_emtypes, type);
+	int type;
+	if (elf->elf_class == ELFCLASS32)
+		type = EGET(EHDR32(elf->ehdr)->e_machine);
+	else
+		type = EGET(EHDR64(elf->ehdr)->e_machine);
+	return type;
 }
 
+const char *get_elfemtype(elfobj *elf)
+{
+	return find_pairtype(elf_emtypes, get_emtype(elf));
+}
+
+
 /* translate elf PT_ defines */
 static pairtype elf_ptypes[] = {
 	QUERY(PT_NULL),
@@ -362,7 +403,7 @@ elfobj *readelf_buffer(const char *filename, char *buffer, size_t buffer_len)
 	if (buffer == NULL || buffer_len < EI_NIDENT)
 		return NULL;
 
-	elf = (elfobj*)malloc(sizeof(*elf));
+	elf = xmalloc(sizeof(*elf));
 	if (elf == NULL)
 		return NULL;
 	memset(elf, 0x00, sizeof(*elf));
@@ -564,7 +605,7 @@ unsigned long pax_pf2hf_flags(unsigned long paxflags)
 	char *pf_flags = pax_short_pf_flags(paxflags);
 	size_t x, len = strlen(pf_flags);
 	for (x = 0; x < len; x++) {
-		switch(pf_flags[x]) {
+		switch (pf_flags[x]) {
 			case 'p':
 				flags |= HF_PAX_PAGEEXEC;
 				break;
