@@ -1,10 +1,10 @@
 /*
- * Copyright 2005-2006 Gentoo Foundation
+ * Copyright 2005-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/porting.h,v 1.23 2006/11/28 04:01:04 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/porting.h,v 1.31 2007/05/23 22:27:27 solar Exp $
  *
- * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
  *
  * Make sure all of the common elf stuff is setup as we expect
  */
@@ -32,6 +32,7 @@
 #include <ctype.h>
 #include <pwd.h>
 #include <fnmatch.h>
+#include <regex.h>
 
 #include <sys/mman.h>
 #include "elf.h"
@@ -40,6 +41,10 @@
 # include <byteswap.h>
 #elif defined(__FreeBSD__)
 # include <sys/endian.h>
+#elif defined(__sun__)
+# include <sys/isa_defs.h>
+#elif defined(__MACH__)
+# include <machine/endian.h>
 #endif
 
 #if defined(__GLIBC__) || defined(__UCLIBC__)
@@ -52,7 +57,7 @@
 # include <elf-hints.h>
 # define __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG _PATH_ELF_HINTS
 #else
-# undef __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG
+# define __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG ""
 #endif
 
 /* bounds checking code will fart on free(NULL) even though that
@@ -109,24 +114,35 @@
 # define __PAX_UTILS_PATH_MAX PATH_MAX
 #endif
 
+/* fall back case for non-Linux hosts ... so lame */
 #if !defined(ELF_DATA)
-# undef __PAX_UTILS_BO
 # if defined(BYTE_ORDER)
-#  define __PAX_UTILS_BO BYTE_ORDER
+#  if BYTE_ORDER == LITTLE_ENDIAN
+#   define ELF_DATA ELFDATA2LSB
+#  elif BYTE_ORDER == BIG_ENDIAN
+#   define ELF_DATA ELFDATA2MSB
+#  else
+#   error "BYTE_ORDER: you fail"
+#  endif
 # elif defined(__BYTE_ORDER)
-#  define __PAX_UTILS_BO __BYTE_ORDER
+#  if __BYTE_ORDER == __LITTLE_ENDIAN
+#   define ELF_DATA ELFDATA2LSB
+#  elif __BYTE_ORDER == __BIG_ENDIAN
+#   define ELF_DATA ELFDATA2BSB
+#  else
+#   error "__BYTE_ORDER: you fail"
+#  endif
 # elif defined(WORDS_LITTLENDIAN)
-#  define __PAX_UTILS_BO LITTLE_ENDIAN
+#  define ELF_DATA ELFDATA2LSB
 # elif defined(WORDS_BIGENDIAN)
-#  define __PAX_UTILS_BO BIG_ENDIAN
+#  define ELF_DATA ELFDATA2MSB
+# elif defined(_LITTLE_ENDIAN)
+#  define ELF_DATA ELFDATA2LSB
+# elif defined(_BIG_ENDIAN)
+#  define ELF_DATA ELFDATA2MSB
 # else
 #  error "no idea what the native byte order is"
 # endif
-# if __PAX_UTILS_BO == LITTLE_ENDIAN
-#  define ELF_DATA ELFDATA2LSB
-# else
-#  define ELF_DATA ELFDATA2MSB
-# endif
 #endif
 
 /*
