@@ -1,16 +1,16 @@
 /*
- * Copyright 2003-2006 Gentoo Foundation
+ * Copyright 2003-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.166 2006/12/11 03:31:54 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.186 2007/08/20 09:54:15 vapier Exp $
  *
- * Copyright 2003-2006 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2004-2006 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2003-2007 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2004-2007 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-#include "paxinc.h"
+static const char *rcsid = "$Id: scanelf.c,v 1.186 2007/08/20 09:54:15 vapier Exp $";
+const char * const argv0 = "scanelf";
 
-static const char *rcsid = "$Id: scanelf.c,v 1.166 2006/12/11 03:31:54 vapier Exp $";
-#define argv0 "scanelf"
+#include "paxinc.h"
 
 #define IS_MODIFIER(c) (c == '%' || c == '#' || c == '+')
 
@@ -27,12 +27,7 @@ static void usage(int status);
 static char **get_split_env(const char *envvar);
 static void parseenv(void);
 static int parseargs(int argc, char *argv[]);
-static char *xstrdup(const char *s);
-static void *xmalloc(size_t size);
-static void *xrealloc(void *ptr, size_t size);
-static void xstrncat(char **dst, const char *src, size_t *curr_len, size_t n);
-#define xstrcat(dst,src,curr_len) xstrncat(dst,src,curr_len,0)
-static inline void xchrcat(char **dst, const char append, size_t *curr_len);
+static int rematch(const char *regex, const char *match, int cflags);
 
 /* variables to control behavior */
 static char match_etypes[126] = "";
@@ -44,6 +39,7 @@ static char scan_archives = 0;
 static char dir_recurse = 0;
 static char dir_crossmount = 1;
 static char show_pax = 0;
+static char show_perms = 0;
 static char show_phdr = 0;
 static char show_textrel = 0;
 static char show_rpath = 0;
@@ -53,6 +49,7 @@ static char show_bind = 0;
 static char show_soname = 0;
 static char show_textrels = 0;
 static char show_banner = 1;
+static char show_endian = 0;
 static char be_quiet = 0;
 static char be_verbose = 0;
 static char be_wewy_wewy_quiet = 0;
@@ -63,7 +60,7 @@ static char *find_section = NULL;
 static char *out_format = NULL;
 static char *search_path = NULL;
 static char fix_elf = 0;
-static char gmatch = 0;
+static char g_match = 0;
 static char use_ldcache = 0;
 
 static char **qa_textrels = NULL;
@@ -71,11 +68,75 @@ static char **qa_execstack = NULL;
 static char **qa_wx_load = NULL;
 
 int match_bits = 0;
+unsigned int match_perms = 0;
 caddr_t ldcache = 0;
 size_t ldcache_size = 0;
 unsigned long setpax = 0UL;
 
+int has_objdump = 0;
 
+static char *getstr_perms(const char *fname);
+static char *getstr_perms(const char *fname)
+{
+	struct stat st;
+	static char buf[8];
+
+	if ((stat(fname, &st)) == (-1))
+		return (char *) "";
+
+	snprintf(buf, sizeof(buf), "%o", st.st_mode);
+
+	return (char *) buf + 2;
+}
+
+/* find the path to a file by name */
+static char *which(const char *fname)
+{
+	static char fullpath[BUFSIZ];
+	char *path, *p;
+
+	path = getenv("PATH");
+	if (!path)
+		return NULL;
+
+	path = xstrdup(path);
+	while ((p = strrchr(path, ':')) != NULL) {
+		snprintf(fullpath, sizeof(fullpath), "%s/%s", p + 1, fname);
+		*p = 0;
+		if (access(fullpath, R_OK) != (-1)) {
+			free(path);
+			return (char *) fullpath;
+		}
+	}
+	free(path);
+	return NULL;
+}
+
+/* 1 on failure. 0 otherwise */
+static int rematch(const char *regex, const char *match, int cflags)
+{
+	regex_t preg;
+	int ret;
+
+	if ((match == NULL) || (regex == NULL))
+		return EXIT_FAILURE;
+
+
+	if ((ret = regcomp(&preg, regex, cflags))) {
+		char err[256];
+
+		if (regerror(ret, &preg, err, sizeof(err)))
+			fprintf(stderr, "regcomp failed: %s", err);
+		else
+			fprintf(stderr, "regcomp failed");
+
+		return EXIT_FAILURE;
+	}
+	ret = regexec(&preg, match, 0, NULL, 0);
+	regfree(&preg);
+
+	return ret;
+}
 
 /* sub-funcs for scanelf_file() */
 static void scanelf_file_get_symtabs(elfobj *elf, void **sym, void **tab)
@@ -275,6 +336,10 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 		return ret;
 }
 
+/*
+ * See if this ELF contains a DT_TEXTREL tag in any of its
+ * PT_DYNAMIC sections.
+ */
 static const char *scanelf_file_textrel(elfobj *elf, char *found_textrel)
 {
 	static const char *ret = "TEXTREL";
@@ -292,7 +357,7 @@ static const char *scanelf_file_textrel(elfobj *elf, char *found_textrel)
 	Elf ## B ## _Phdr *phdr = PHDR ## B (elf->phdr); \
 	Elf ## B ## _Off offset; \
 	for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
-		if (EGET(phdr[i].p_type) != PT_DYNAMIC) continue; \
+		if (EGET(phdr[i].p_type) != PT_DYNAMIC || EGET(phdr[i].p_filesz) == 0) continue; \
 		offset = EGET(phdr[i].p_offset); \
 		if (offset >= elf->len - sizeof(Elf ## B ## _Dyn)) continue; \
 		dyn = DYN ## B (elf->data + offset); \
@@ -314,6 +379,12 @@ static const char *scanelf_file_textrel(elfobj *elf, char *found_textrel)
 	else
 		return "   -   ";
 }
+
+/*
+ * Scan the .text section to see if there are any relocations in it.
+ * Should rewrite this to check PT_LOAD sections that are marked
+ * Executable rather than the section named '.text'.
+ */
 static char *scanelf_file_textrels(elfobj *elf, char *found_textrels, char *found_textrel)
 {
 	unsigned long s, r, rmax;
@@ -390,7 +461,7 @@ static char *scanelf_file_textrels(elfobj *elf, char *found_textrels, char *foun
 			if (sym && sym->st_name) \
 				printf("%s", (char*)(elf->data + EGET(strtab->sh_offset) + EGET(sym->st_name))); \
 			else \
-				printf("(memory/fake?)"); \
+				printf("(memory/data?)"); \
 			printf(" [0x%lX]", (unsigned long)r_offset); \
 			/* now try to find the closest symbol that this rel is probably in */ \
 			sym = SYM ## B (elf->data + EGET(symtab->sh_offset)); \
@@ -404,11 +475,37 @@ static char *scanelf_file_textrels(elfobj *elf, char *found_textrels, char *foun
 				++sym; \
 			} \
 			printf(" in "); \
-			if (func && func->st_name) \
-				printf("%s", (char*)(elf->data + EGET(strtab->sh_offset) + EGET(func->st_name))); \
-			else \
-				printf("(NULL: fake?)"); \
+			if (func && func->st_name) { \
+				const char *func_name = elf->data + EGET(strtab->sh_offset) + EGET(func->st_name); \
+				if (r_offset > EGET(func->st_size)) \
+					printf("(optimized out: previous %s)", func_name); \
+				else \
+					printf("%s", func_name); \
+			} else \
+				printf("(optimized out)"); \
 			printf(" [0x%lX]\n", (unsigned long)offset_tmp); \
+			if (be_verbose && has_objdump) { \
+				char *sysbuf; \
+				size_t syslen; \
+				const char sysfmt[] = "objdump -r -R -d -w -l --start-address=0x%lX --stop-address=0x%lX %s | grep --color -i -C 3 '.*[[:space:]]%lX:[[:space:]]*R_.*'\n"; \
+				syslen = sizeof(sysfmt) + strlen(elf->filename) + 3 * sizeof(unsigned long) + 1; \
+				sysbuf = xmalloc(syslen); \
+				if (sysbuf) { \
+					Elf ## B ## _Addr end_addr = offset_tmp + EGET(func->st_size); \
+					if (end_addr < r_offset) \
+						/* not uncommon when things are optimized out */ \
+						end_addr = r_offset + 0x100; \
+					snprintf(sysbuf, syslen, sysfmt, \
+						(unsigned long)offset_tmp, \
+						(unsigned long)end_addr, \
+						elf->filename, \
+						(unsigned long)r_offset); \
+					fflush(stdout); \
+					system(sysbuf); \
+					fflush(stdout); \
+					free(sysbuf); \
+				} \
+			} \
 		} \
 	} }
 	SHOW_TEXTRELS(32)
@@ -437,7 +534,7 @@ static void rpath_security_checks(elfobj *elf, char *item, const char *dt_type)
 			if (fstat(elf->fd, &st) != -1)
 				if ((st.st_mode & S_ISUID) || (st.st_mode & S_ISGID))
 					warnf("Security problem with %s='%s' in %s with mode set of %o",
-					      dt_type, item, elf->filename, st.st_mode & 07777);
+					      dt_type, item, elf->filename, (unsigned int) st.st_mode & 07777);
 			break;
 		default:
 			warnf("Maybe? sec problem with %s='%s' in %s", dt_type, item, elf->filename);
@@ -467,7 +564,7 @@ static void scanelf_file_rpath(elfobj *elf, char *found_rpath, char **ret, size_
 		/* Scan all the program headers */ \
 		for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
 			/* Just scan dynamic headers */ \
-			if (EGET(phdr[i].p_type) != PT_DYNAMIC) continue; \
+			if (EGET(phdr[i].p_type) != PT_DYNAMIC || EGET(phdr[i].p_filesz) == 0) continue; \
 			offset = EGET(phdr[i].p_offset); \
 			if (offset >= elf->len - sizeof(Elf ## B ## _Dyn)) continue; \
 			/* Just scan dynamic RPATH/RUNPATH headers */ \
@@ -727,7 +824,7 @@ static const char *scanelf_file_needed_lib(elfobj *elf, char *found_needed, char
 		Elf ## B ## _Shdr *strtbl = SHDR ## B (strtbl_void); \
 		Elf ## B ## _Off offset; \
 		for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
-			if (EGET(phdr[i].p_type) != PT_DYNAMIC) continue; \
+			if (EGET(phdr[i].p_type) != PT_DYNAMIC || EGET(phdr[i].p_filesz) == 0) continue; \
 			offset = EGET(phdr[i].p_offset); \
 			if (offset >= elf->len - sizeof(Elf ## B ## _Dyn)) continue; \
 			dyn = DYN ## B (elf->data + offset); \
@@ -749,7 +846,7 @@ static const char *scanelf_file_needed_lib(elfobj *elf, char *found_needed, char
 						} \
 						*found_needed = 1; \
 					} else { \
-						if (!strncmp(find_lib, needed, strlen( !gmatch ? needed : find_lib))) { \
+						if (!strncmp(find_lib, needed, strlen( !g_match ? needed : find_lib))) { \
 							*found_lib = 1; \
 							return (be_wewy_wewy_quiet ? NULL : needed); \
 						} \
@@ -802,7 +899,7 @@ static char *scanelf_file_bind(elfobj *elf, char *found_bind)
 		Elf ## B ## _Phdr *phdr = PHDR ## B (elf->phdr); \
 		Elf ## B ## _Off offset; \
 		for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
-			if (EGET(phdr[i].p_type) != PT_DYNAMIC) continue; \
+			if (EGET(phdr[i].p_type) != PT_DYNAMIC || EGET(phdr[i].p_filesz) == 0) continue; \
 			dynamic = 1; \
 			offset = EGET(phdr[i].p_offset); \
 			if (offset >= elf->len - sizeof(Elf ## B ## _Dyn)) continue; \
@@ -854,7 +951,7 @@ static char *scanelf_file_soname(elfobj *elf, char *found_soname)
 		if (ehdr->e_type != ET_DYN) \
 			return NULL; \
 		for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
-			if (EGET(phdr[i].p_type) != PT_DYNAMIC) continue; \
+			if (EGET(phdr[i].p_type) != PT_DYNAMIC || EGET(phdr[i].p_filesz) == 0) continue; \
 			offset = EGET(phdr[i].p_offset); \
 			if (offset >= elf->len - sizeof(Elf ## B ## _Dyn)) continue; \
 			dyn = DYN ## B (elf->data + offset); \
@@ -878,6 +975,7 @@ static char *scanelf_file_soname(elfobj *elf, char *found_soname)
 
 	return NULL;
 }
+
 static char *scanelf_file_sym(elfobj *elf, char *found_sym)
 {
 	unsigned long i;
@@ -909,13 +1007,13 @@ static char *scanelf_file_sym(elfobj *elf, char *found_sym)
 					continue; \
 				} \
 				/* debug display ... show all symbols and some extra info */ \
-				if (*ret == '*') { \
-					printf("%s(%s) %5lX %15s %s\n", \
+				if (g_match ? rematch(ret, symname, REG_EXTENDED) == 0 : *ret == '*') { \
+					printf("%s(%s) %5lX %15s %s %s\n", \
 					       ((*found_sym == 0) ? "\n\t" : "\t"), \
 					       elf->base_filename, \
 					       (unsigned long)sym->st_size, \
 					       get_elfstttype(sym->st_info), \
-					       symname); \
+					       sym->st_shndx == SHN_UNDEF ? "U" : "D", symname); \
 					*found_sym = 1; \
 				} else { \
 					/* allow the user to specify a comma delimited list of symbols to search for */ \
@@ -1028,7 +1126,7 @@ static int scanelf_elfobj(elfobj *elf)
 	/* init output buffer */
 	if (!out_buffer) {
 		out_len = sizeof(char) * 80;
-		out_buffer = (char*)xmalloc(out_len);
+		out_buffer = xmalloc(out_len);
 	}
 	*out_buffer = '\0';
 
@@ -1049,6 +1147,7 @@ static int scanelf_elfobj(elfobj *elf)
 			case 'e': prints("STK/REL/PTL "); break;
 			case 't': prints("TEXTREL "); break;
 			case 'r': prints("RPATH "); break;
+			case 'M': prints("CLASS "); break;
 			case 'n': prints("NEEDED "); break;
 			case 'i': prints("INTERP "); break;
 			case 'b': prints("BIND "); break;
@@ -1058,6 +1157,8 @@ static int scanelf_elfobj(elfobj *elf)
 			case 'T': prints("TEXTRELS "); break;
 			case 'k': prints("SECTION "); break;
 			case 'a': prints("ARCH "); break;
+			case 'O': prints("PERM "); break;
+			case 'D': prints("ENDIAN "); break;
 			default: warnf("'%c' has no title ?", out_format[i]);
 			}
 		}
@@ -1117,6 +1218,9 @@ static int scanelf_elfobj(elfobj *elf)
 		case 't': out = scanelf_file_textrel(elf, &found_textrel); break;
 		case 'T': out = scanelf_file_textrels(elf, &found_textrels, &found_textrel); break;
 		case 'r': scanelf_file_rpath(elf, &found_rpath, &out_buffer, &out_len); break;
+		case 'M': out = get_elfeitype(EI_CLASS, elf->data[EI_CLASS]); break;
+		case 'D': out = get_endian(elf); break;
+		case 'O': out = getstr_perms(elf->filename); break;
 		case 'n':
 		case 'N': out = scanelf_file_needed_lib(elf, &found_needed, &found_lib, (out_format[i]=='N'), &out_buffer, &out_len); break;
 		case 'i': out = scanelf_file_interp(elf, &found_interp); break;
@@ -1180,7 +1284,7 @@ static int scanelf_elf(const char *filename, int fd, size_t len)
 		strncpy(sbuf, match_etypes, sizeof(sbuf));
 		if (strchr(match_etypes, ',') != NULL) {
 			char *p;
-			while((p = strrchr(sbuf, ',')) != NULL) {
+			while ((p = strrchr(sbuf, ',')) != NULL) {
 				*p = 0;
 				if (etype_lookup(p+1) == get_etype(elf))
 					goto label_ret;
@@ -1241,6 +1345,10 @@ static int scanelf_file(const char *filename, const struct stat *st_cache)
 		return 1;
 	}
 
+	if (match_perms) {
+		if ((st->st_mode | match_perms) != st->st_mode)
+			return 1;
+	}
 	if ((fd=open(filename, (fix_elf ? O_RDWR : O_RDONLY))) == -1)
 		return 1;
 
@@ -1347,7 +1455,7 @@ static int load_ld_cache_config(int i, const char *fname)
 		if ((p = strchr(path, '\n')) != NULL)
 			*p = 0;
 #ifdef __linux__
-		// recursive includes of the same file will make this segfault.
+		/* recursive includes of the same file will make this segfault. */
 		if ((memcmp(path, "include", 7) == 0) && isblank(path[7])) {
 			glob64_t gl;
 			size_t x;
@@ -1412,7 +1520,7 @@ static int load_ld_cache_config(int i, const char *fname)
 		return i;
 	}
 
-	b = (char*)malloc(hdr.dirlistlen+1);
+	b = xmalloc(hdr.dirlistlen + 1);
 	if (fread(b, 1, hdr.dirlistlen+1, fp) != hdr.dirlistlen+1) {
 		fclose(fp);
 		free(b);
@@ -1444,7 +1552,7 @@ static int load_ld_cache_config(int i, const char *fname)
 #endif
 
 /* scan /etc/ld.so.conf for paths */
-static void scanelf_ldpath()
+static void scanelf_ldpath(void)
 {
 	char scan_l, scan_ul, scan_ull;
 	int i = 0;
@@ -1468,7 +1576,7 @@ static void scanelf_ldpath()
 }
 
 /* scan env PATH for paths */
-static void scanelf_envpath()
+static void scanelf_envpath(void)
 {
 	char *path, *p;
 
@@ -1485,8 +1593,9 @@ static void scanelf_envpath()
 	free(path);
 }
 
-/* usage / invocation handling functions */
-#define PARSE_FLAGS "plRmyAXz:xetrnLibSs:k:gN:TaqvF:f:o:E:M:BhV"
+
+/* usage / invocation handling functions */ /* Free Flags: c d j u w C G H I J K P Q U W Y Z */
+#define PARSE_FLAGS "plRmyAXz:xetrnLibSs:k:gN:TaqvF:f:o:E:M:DO:BhV"
 #define a_argument required_argument
 static struct option const long_opts[] = {
 	{"path",      no_argument, NULL, 'p'},
@@ -1513,6 +1622,8 @@ static struct option const long_opts[] = {
 	{"textrels",  no_argument, NULL, 'T'},
 	{"etype",      a_argument, NULL, 'E'},
 	{"bits",       a_argument, NULL, 'M'},
+	{"endian",    no_argument, NULL, 'D'},
+	{"perms",      a_argument, NULL, 'O'},
 	{"all",       no_argument, NULL, 'a'},
 	{"quiet",     no_argument, NULL, 'q'},
 	{"verbose",   no_argument, NULL, 'v'},
@@ -1550,6 +1661,8 @@ static const char *opts_help[] = {
 	"Locate cause of TEXTREL",
 	"Print only ELF files matching etype ET_DYN,ET_EXEC ...",
 	"Print only ELF files matching numeric bits",
+	"Print Endianness",
+	"Print only ELF files matching octal permissions",
 	"Print all scanned info (-x -e -t -r -b)\n",
 	"Only output 'bad' things",
 	"Be verbose (can be specified more than once)",
@@ -1577,22 +1690,7 @@ static void usage(int status)
 			printf("  -%c, --%-7s <arg> * %s\n", long_opts[i].val,
 			       long_opts[i].name, opts_help[i]);
 
-	if (status != EXIT_SUCCESS)
-		exit(status);
-
-	puts("\nThe format modifiers for the -F option are:");
-	puts(" F Filename \tx PaX Flags \te STACK/RELRO");
-	puts(" t TEXTREL  \tr RPATH     \tn NEEDED");
-	puts(" i INTERP   \tb BIND      \ts symbol");
-	puts(" N library  \to Type      \tT TEXTRELs");
-	puts(" S SONAME   \tk section   \ta arch");
-	puts(" p filename (with search path removed)");
-	puts(" f filename (short name/basename)");
-	puts("Prefix each modifier with '%' (verbose) or '#' (silent)");
-
-	puts("\nELF Etypes:");
-	print_etypes(stdout);
-
+	puts("\nFor more information, see the scanelf(1) manpage");
 	exit(status);
 }
 
@@ -1631,6 +1729,16 @@ static int parseargs(int argc, char *argv[])
 			break;
 		case 'M':
 			match_bits = atoi(optarg);
+			if (match_bits == 0) {
+				if (strcmp(optarg, "ELFCLASS32") == 0)
+					match_bits = 32;
+				if (strcmp(optarg, "ELFCLASS64") == 0)
+					match_bits = 64;
+			}
+			break;
+		case 'O':
+			if (sscanf(optarg, "%o", &match_perms) == (-1))
+				match_bits = 0;
 			break;
 		case 'o': {
 			if (freopen(optarg, "w", stdout) == NULL)
@@ -1644,7 +1752,7 @@ static int parseargs(int argc, char *argv[])
 		case 's': {
 			if (find_sym) warn("You prob don't want to specify -s twice");
 			find_sym = optarg;
-			versioned_symname = (char*)xmalloc(sizeof(char) * (strlen(find_sym)+1+1));
+			versioned_symname = xmalloc(sizeof(char) * (strlen(find_sym)+1+1));
 			sprintf(versioned_symname, "%s@", find_sym);
 			break;
 		}
@@ -1663,8 +1771,8 @@ static int parseargs(int argc, char *argv[])
 			unsigned long flags = (PF_NOEMUTRAMP | PF_NORANDEXEC);
 			size_t x;
 
-			for (x = 0 ; x < strlen(optarg); x++) {
-				switch(optarg[x]) {
+			for (x = 0; x < strlen(optarg); x++) {
+				switch (optarg[x]) {
 					case 'p':
 					case 'P':
 						do_pax_state(optarg[x], PAGEEXEC);
@@ -1702,7 +1810,7 @@ static int parseargs(int argc, char *argv[])
 					setpax = flags;
 			break;
 		}
-		case 'g': gmatch = 1; break;
+		case 'g': g_match = 1; break;
 		case 'L': use_ldcache = 1; break;
 		case 'y': scan_symlink = 0; break;
 		case 'A': scan_archives = 1; break;
@@ -1723,8 +1831,8 @@ static int parseargs(int argc, char *argv[])
 		case 'T': show_textrels = 1; break;
 		case 'q': be_quiet = 1; break;
 		case 'v': be_verbose = (be_verbose % 20) + 1; break;
-		case 'a': show_pax = show_phdr = show_textrel = show_rpath = show_bind = 1; break;
-
+		case 'a': show_perms = show_pax = show_phdr = show_textrel = show_rpath = show_bind = show_endian = 1; break;
+		case 'D': show_endian = 1; break;
 		case ':':
 			err("Option '%c' is missing parameter", optopt);
 		case '?':
@@ -1733,12 +1841,15 @@ static int parseargs(int argc, char *argv[])
 			err("Unhandled option '%c'; please report this", i);
 		}
 	}
-
+	if (show_textrels && be_verbose) {
+		if (which("objdump") != NULL)
+			has_objdump = 1;
+	}
 	/* let the format option override all other options */
 	if (out_format) {
 		show_pax = show_phdr = show_textrel = show_rpath = \
 		show_needed = show_interp = show_bind = show_soname = \
-		show_textrels = 0;
+		show_textrels = show_perms = show_endian = 0;
 		for (i = 0; out_format[i]; ++i) {
 			if (!IS_MODIFIER(out_format[i])) continue;
 
@@ -1754,6 +1865,9 @@ static int parseargs(int argc, char *argv[])
 			case 'N': break;
 			case 'o': break;
 			case 'a': break;
+			case 'M': break;
+			case 'D': show_endian = 1; break;
+			case 'O': show_perms = 1; break;
 			case 'x': show_pax = 1; break;
 			case 'e': show_phdr = 1; break;
 			case 't': show_textrel = 1; break;
@@ -1772,9 +1886,11 @@ static int parseargs(int argc, char *argv[])
 	/* construct our default format */
 	} else {
 		size_t fmt_len = 30;
-		out_format = (char*)xmalloc(sizeof(char) * fmt_len);
+		out_format = xmalloc(sizeof(char) * fmt_len);
 		if (!be_quiet)     xstrcat(&out_format, "%o ", &fmt_len);
 		if (show_pax)      xstrcat(&out_format, "%x ", &fmt_len);
+		if (show_perms)    xstrcat(&out_format, "%O ", &fmt_len);
+		if (show_endian)   xstrcat(&out_format, "%D ", &fmt_len);
 		if (show_phdr)     xstrcat(&out_format, "%e ", &fmt_len);
 		if (show_textrel)  xstrcat(&out_format, "%t ", &fmt_len);
 		if (show_rpath)    xstrcat(&out_format, "%r ", &fmt_len);
@@ -1851,7 +1967,8 @@ static char **get_split_env(const char *envvar)
 	 * the envvals array of strings */
 	return envvals;
 }
-static void parseenv()
+
+static void parseenv(void)
 {
 	qa_textrels = get_split_env("QA_TEXTRELS");
 	qa_execstack = get_split_env("QA_EXECSTACK");
@@ -1859,7 +1976,7 @@ static void parseenv()
 }
 
 #ifdef __PAX_UTILS_CLEANUP
-static void cleanup()
+static void cleanup(void)
 {
 	free(out_format);
 	free(qa_textrels);
@@ -1887,51 +2004,6 @@ int main(int argc, char *argv[])
 }
 
 
-
-/* utility funcs */
-static char *xstrdup(const char *s)
-{
-	char *ret = strdup(s);
-	if (!ret) err("Could not strdup(): %s", strerror(errno));
-	return ret;
-}
-static void *xmalloc(size_t size)
-{
-	void *ret = malloc(size);
-	if (!ret) err("Could not malloc() %li bytes", (unsigned long)size);
-	return ret;
-}
-static void *xrealloc(void *ptr, size_t size)
-{
-	void *ret = realloc(ptr, size);
-	if (!ret) err("Could not realloc() %li bytes", (unsigned long)size);
-	return ret;
-}
-static void xstrncat(char **dst, const char *src, size_t *curr_len, size_t n)
-{
-	size_t new_len;
-
-	new_len = strlen(*dst) + strlen(src);
-	if (*curr_len <= new_len) {
-		*curr_len = new_len + (*curr_len / 2);
-		*dst = realloc(*dst, *curr_len);
-		if (!*dst)
-			err("could not realloc() %li bytes", (unsigned long)*curr_len);
-	}
-
-	if (n)
-		strncat(*dst, src, n);
-	else
-		strcat(*dst, src);
-}
-static inline void xchrcat(char **dst, const char append, size_t *curr_len)
-{
-	static char my_app[2];
-	my_app[0] = append;
-	my_app[1] = '\0';
-	xstrcat(dst, my_app, curr_len);
-}
-
 /* Match filename against entries in matchlist, return TRUE
  * if the file is listed */
 static int file_matches_list(const char *filename, char **matchlist)
